package BIDM::ClassDefinition;

=head1 NAME

BIDM::ClassDefinition

=head1 DESCRIPTION

Defines, with the help of a class definition file, the attributes and relationships

=cut

use strict;

use BIDM::MemberMixin;
use vars qw(@ISA);

@ISA = qw(BIDM::MemberMixin);

use BIDM::Attribute;
use BIDM::Relationship;

use File::Find;
use FileHandle;
use DirHandle;

my $ClassDirectoryMap;

sub new
{
    my($class, $bidm_class) = @_;
    my($self);

    $self = {
	class => $bidm_class,
	attribute_list => [],
	attribute_map => {},
	lc_attribute_map => {},
	relationship_list => [],
	relationship_map => {},
	lc_relationship_map => {},
    };

    bless($self, $class);
    $self->load_class_info();

    return $self;
}

sub lookup_attribute
{
    my($self, $name) = @_;
    my($attribute);

    $attribute = $self->attribute_map()->{$name};
    return $attribute if $attribute;

    $attribute = $self->lc_attribute_map()->{lc($name)};
    return $attribute if $attribute;

#    die "Attribute $name not found in class ", $self->class(), "\n";
    return undef;
}

sub lookup_relationship
{
    my($self, $name) = @_;
    my($relationship);

    $relationship = $self->relationship_map()->{$name};
    return $relationship if $relationship;

    $relationship = $self->lc_relationship_map()->{lc($name)};
    return $relationship if $relationship;

    #die "Relationship $name not found in class ", $self->class(), "\n";
    return undef;
}

sub load_class_info
{
    my($self) = @_;
    my(@dirs, $dir);

    @dirs = $self->find_class_directories($self->class());

    for $dir (@dirs)
    {
#	print "Load from <$dir>\n";
	$self->load_class_from_directory($dir);
    }
}

sub load_class_from_directory
{
    my($self, $dir) = @_;
    my($fh);

    $fh = new FileHandle("<$dir/attributes");
    if ($fh)
    {
	$self->load_attributes($fh);
	$fh->close();
    }

    $fh = new FileHandle("<$dir/relationships");
    if ($fh)
    {
	$self->load_relationships($fh);
	$fh->close();
    }
}

sub load_attributes
{
    my($self, $fh) = @_;

#    print "Loading attributes for ", $self->class(), "\n";
    while (<$fh>)
    {
	chop;
	s/^\s+//;
	s/\s+$//;
	next if /^#/;
	if (/^%attribute\s+(\S+)/)
	{
	    my($attr);
	    $a = new BIDM::Attribute($1);
	    $a->load_from_fh($fh);
	    $self->add_attribute($a);
	}
    }
    
}
sub load_relationships
{
    my($self, $fh) = @_;

#    print "Loading relationships for ", $self->class(), "\n";
    while (<$fh>)
    {
	chop;
	s/^\s+//;
	s/\s+$//;
	next if /^#/;
	if (/^%relationship\s+(\S+)/)
	{
	    my($attr);
	    $a = new BIDM::Relationship($1);
	    $a->load_from_fh($fh);
	    $self->add_relationship($a);
	}
    }
    
}

sub add_attribute
{
    my($self, $attr) = @_;
    my($name, $lcname);

    $name = $attr->name();
    $lcname = lc($name);

    push(@{$self->attribute_list()}, $attr);
    $self->attribute_map()->{$name} = $attr;
    $self->lc_attribute_map()->{$lcname} = $attr;
}

sub add_relationship
{
    my($self, $attr) = @_;
    my($name, $lcname);

    $name = $attr->name();
    $lcname = lc($name);

    push(@{$self->relationship_list()}, $attr);
    $self->relationship_map()->{$name} = $attr;
    $self->lc_relationship_map()->{$lcname} = $attr;
}

sub find_class_directories
{
    my($self, $class) = @_;
    my($map, $dirs);

    if (!ref($ClassDirectoryMap))
    {
	$ClassDirectoryMap = $self->build_directory_map();
    }

#    print "map=$ClassDirectoryMap\n";
    $dirs = $ClassDirectoryMap->{$class};
    if (ref($dirs))
    {
	return @$dirs;
    }
    else
    {
#	print "NO dirs for $class\n";
	return ();
    }
}

sub build_directory_map
{
    my($self) = @_;
    my($dir, $map);

    $map = {};

    for $dir (@BIDM::ClassDefinitionPath)
    {
	$self->_search_topdir($map, $dir);
    }

    return $map;
}

sub _search_topdir
{
    my($self, $map, $topdir) = @_;
    my($model, $dir);

    for $model (@BIDM::DataModels)
    {
	if (-d "$topdir/$model")
	{
	    $self->_search_model($map, $topdir, $model);
	}
    }
}

sub _search_model
{
    my($self, $map, $topdir, $model) = @_;
    my($dir, $dh, $path);

    $path = "$topdir/$model";

    $dh = new DirHandle($path);
    if (!$dh)
    {
	return;
    }

    while (defined($_ = $dh->read()))
    {
	next if /^\./;
#	print "model found $_\n";
	push(@{$map->{$_}}, "$path/$_");
    }
    $dh->close();
}

sub find_class_directories_with_find
{
    my($self, $class) = @_;
    my($list);

    $list = [];
#    print "Finding files in ", join(":", @BIDM::ClassDefinitionPath), "\n";
    find(sub { $self->match_file($class, $list); }, @BIDM::ClassDefinitionPath);
    return(@$list);
}

sub match_file
{
    my($self, $class, $list) = @_;
#    print "match_file dir=$File::Find::dir file=$_\n";
    if ($_ eq $class && -d $_)
    {
	push(@$list, "$File::Find::dir/$_");
	$File::Find::prune = 1;
#	print "found it, pruning\n";
    }
}

sub class { shift->_elem('class', @_); }
sub attribute_list { shift->_elem('attribute_list', @_); }
sub attribute_map { shift->_elem('attribute_map', @_); }
sub lc_attribute_map { shift->_elem('lc_attribute_map', @_); }
sub relationship_list { shift->_elem('relationship_list', @_); }
sub relationship_map { shift->_elem('relationship_map', @_); }
sub lc_relationship_map { shift->_elem('lc_relationship_map', @_); }

1;
