package BIDM::RIGObject;

=head1 NAME

BIDM::RIGObject

=head1 DESCRIPTION

=cut

use strict;
use vars qw(@ISA $AUTOLOAD);

use BIDM::MemberMixin;
use BIDM::RIGObject::Asset;
use BIDM::RIGObject::Organization;
use BIDM::RIGObject::Library;

require AutoLoader;

@ISA = qw(BIDM::MemberMixin AutoLoader);

my $ClassName = "RIGObject";
my $ClassDefinition = new BIDM::ClassDefinition($ClassName);

=head2 $obj = BIDM::RIGObject::object_for_class($class)

Look up the RIG class for this classname, and instantiate
an instance of it.

=cut

sub object_for_class
{
    my($class) = @_;

    $class = "\L$class";
    if ($class eq "asset")
    {
	return BIDM::RIGObject::Asset->new();
    }
    elsif ($class eq "organization")
    {
	return BIDM::RIGObject::Organization->new();
    }
    elsif ($class eq "library")
    {
	return BIDM::RIGObject::Library->new();
    }
    else
    {
	#die "Class `$class' not defined\n";
	return undef;
    }
}

sub new
{
    my($class) = @_;
    my($self);

    $self = {
	attributes => {},
	relationships => {},
	user_hash => {},
    };
    bless($self, $class);

    $self->class_name("RIGObject");
    
    return $self;

}

=head2 $obj->isa($class)

Returns true if this object is a (descendant) member of $class

=cut

sub isa
{
    my($self, $class) = @_;

    return $self->class_name() eq $class;
}

=head2 $obj->handle_tag_attribute($name, $value)

Handle an attribute tag, as generated during the tag parsing process.

Look up the attribute object for this name, and pass the tag's name 
and value through to that object. This is actually done using the
RIGObject::set_attribute method.

=cut  

sub handle_tag_attribute
{
    my($self, $name, $value) = @_;
    my($cname, $attribute_def);

    $self->set_attribute($name, $value);
}

=head2 $obj->set_attribute($name, $value)

Set the RIG object's attribute named $name to $value. We do this by
looking up the attribute object for $name, and invoking it's
set_object_attribute method, passing this object through. We do this
to allow the attribute object (which is what contains the information needed
to do typechecking on attributes) to actually do the value setting.

=cut

sub set_attribute
{
    my($self, $name, $value) = @_;
    my($attribute_obj);

#    print "Attribute $name on $self: $value\n";

    $attribute_obj = $self->lookup_attribute($name);

    if (!defined($attribute_obj))
    {
	warn "Attribute $name not defined on ", $self->class_name(), "\n";
	return undef;
    }

#    $attribute_obj->print();

    $attribute_obj->set_object_attribute($self, $value);
#    $self->print();
}

=head2 $obj->handle_tag_inline_relationship($name, $seq_num, $rel_class, $rel_attribute, $value)

Parsing support for RIG object relationships.

The relationship pointer in an object points to either a URL containing    
the pointed-to object, or a BIDM::RIGObject subclass which is the 
target of the relationship. The latter will occur with inlined
relationships. The former can be expanded with a spider, if desired.

=cut  

sub handle_tag_inline_relationship
{
    my($self, $name, $seq_num, $rel_class, $rel_attribute,
       $value) = @_;
    my($cname, $target, $rel_obj);

#    print "----have inline rel name=$name seq=$seq_num class=$rel_class attr=$rel_attribute\n";
#    print "value=$value\n";

    #
    # Look up the relationship object for this relationship.
    #

    $rel_obj = $self->lookup_relationship($name);
    if (!$rel_obj)
    {
	die "Relation $name not found\n";
    }
    $cname = $rel_obj->name();

#    print "Got rel_obj $cname $rel_obj\n";

    #
    # Look into the relationship value to see if we already have one.
    #

    my $rel_hash = $self->relationships()->{$cname};
    if (!defined($rel_hash))
    {
	$rel_hash = {};
	$self->relationships()->{$cname} = $rel_hash;
    }

    #
    # Have we created this instance yet? If not, create one
    #
    # Note that rel_inst is the that is the target of this relationship.
    #
    my $rel_inst = $rel_hash->{$seq_num};

    if (!defined($rel_inst))
    {
	$rel_inst = object_for_class($rel_class);
#	print "created inst $rel_inst for class $rel_class seq $seq_num\n";
	return undef unless defined($rel_inst);
	$rel_hash->{$seq_num} = $rel_inst;
    }
    else
    {
#	print "already had $rel_inst for $rel_class seq $seq_num\n";
    }

    #
    # Set the attribute on it.
    #

    eval { $rel_inst->set_attribute($rel_attribute, $value); };
    if ($@)
    {
	print "Hm, set_attr $rel_inst failed with $@\n";
    }

    return;

    $cname = $self->map_relationship_name($name);

    if (!defined($cname))
    {
	warn "Relationship $name not defined on ", $self->class_name(), "\n";
	return undef;
    }

#    print "Relationship $cname on $self: $value\n";

}

sub handle_tag_relationship
{
    my($self, $name, $rel) = @_;
    my($cname, $target, $rel_obj);
    my($seq_num);

    #
    # Just define a seq number, so we can reuse some of the code below
    # and the structure for inlined relationships.
    #
    $seq_num = 1;

#    print "----have inline rel name=$name seq=$seq_num class=$rel_class attr=$rel_attribute\n";
#    print "value=$value\n";

    #
    # Look up the relationship object for this relationship.
    #

    $rel_obj = $self->lookup_relationship($name);
    $cname = $rel_obj->name();

#    print "Got rel_obj $cname $rel_obj\n";

    #
    # Look into the relationship value to see if we already have one.
    #

    my $rel_hash = $self->relationships()->{$cname};
    if (!defined($rel_hash))
    {
	$rel_hash = {};
	$self->relationships()->{$cname} = $rel_hash;
    }

    $rel_hash->{$seq_num} = $rel;

    return;
}

=head2 $obj->relationship_link($type, $name, $link_obj)

Add a relationship which is implememtned as a html LINK. $link_obj is
the instance of BIDM::Tag::Link with the link info in it,. $type is
either 'rev' or 'rel'.

=cut
    
sub attribute_link
{
    my($self, $type, $name, $link_obj) = @_;
    my($cname);

    $cname = $self->map_attribute_name($name);

    if (!defined($cname))
    {
	warn "Attribute $name not defined on ", $self->class_name(), "\n";
	return undef;
    }

    print "Attribute type=$type $cname on $self: $link_obj\n";

}

sub relationship_link
{
    my($self, $type, $name, $rel_class, $link_obj) = @_;
    my($cname);

    $cname = $self->map_relationship_name($name);

    if (!defined($cname))
    {
	warn "Relationship $name not defined on ", $self->class_name(), "\n";
	return undef;
    }

    print "Relationship type=$type $cname on $self: class=$rel_class $link_obj\n";

}

sub lookup_attribute
{
    my($self, $name) = @_;
    my($attribute);

    $attribute = $self->class_definition()->lookup_attribute($name);

    return $attribute;
}

sub lookup_relationship
{
    my($self, $name) = @_;
    my($relationship);

    $relationship = $self->class_definition()->lookup_relationship($name);

    return $relationship;
}

sub print
{
    my($self) = @_;
    my($class);
    my($k, $v, $a);
    
    $class = $self->class_name();
    print "RIG Object $class\n";
    if (defined($a = $self->attributes()))
    {
	print "Attributes:\n";
	for $k (sort keys(%$a))
	{
	    $v = $a->{$k};
	    print "    $k = $v\n";
	}
    }
    if (defined($a = $self->relationships()))
    {
	print "Relationships:\n";
	for $k (sort keys(%$a))
	{
	    $v = $a->{$k};
	    print "    $k = $v\n";
	}
    }
}

sub class_name { shift->_elem('class_name', @_); }
sub class_definition { shift->_elem('class_definition', @_); }

sub attributes { shift->_elem('attributes', @_); }
sub relationships { shift->_elem('relationships', @_); }

sub user_hash { shift->_elem('user_hash', @_); }

sub AUTOLOAD
{
    my($self, @args) = @_;
    my($attr_obj, $rel_obj, $name);

    ($name = $AUTOLOAD) =~ s/.*:://;
    
#    print "RIG autoload: $self <@args> $name\n";

    if ($attr_obj = $self->lookup_attribute($name))
    {
	return $self->access_attribute($attr_obj, @args);
    }
    elsif ($rel_obj = $self->lookup_relationship($name))
    {
	return $self->access_relationship($rel_obj, @args);
    }
    else
    {
	die "Attribute or relationship $name not found\n";
    }
}

sub DESTROY
{
}

sub access_attribute
{
    my($self, $attr_obj, $val) = @_;

    if ($val)
    {
	die "access_attribute: setting of values not yet implemented\n";
    }

    return $attr_obj->get_object_attribute($self);
}

sub access_relationship
{
    my($self, $rel_obj, $val) = @_;

    if ($val)
    {
	die "access_relationship: setting of values not yet implemented\n";
    }

    return $rel_obj->get_object_relationship($self);
}

1;

__END__
